unit UnitMain;

interface

uses
  System.SysUtils, System.Classes, JS, Web,

  generics.Collections,

  WEBLib.Graphics, WEBLib.Controls,
  WEBLib.Forms, WEBLib.Dialogs, Vcl.StdCtrls, WEBLib.StdCtrls, Vcl.Controls,
  WEBLib.WebCtrls,

  Unit1,
  Unit2,
  Unit3,
  Unit4;

type
  TFormList = class(TObjectDictionary<string, TWebHTMLDiv>);

  TMainForm = class(TWebForm)
    TitleDiv: TWebHTMLDiv;
    WebButton1: TWebButton;
    WebButton2: TWebButton;
    FormsDiv: TWebHTMLDiv;
    WebButton3: TWebButton;
    WebButton4: TWebButton;
    procedure WebButton1Click(Sender: TObject);
    procedure WebFormCreate(Sender: TObject);
    procedure WebButton2Click(Sender: TObject);
    procedure WebButton3Click(Sender: TObject);
    procedure WebButton4Click(Sender: TObject);
  private
    { Private declarations }
    FFormList: TFormList;
    FCurrentFormDiv: TWebHTMLDiv;
    FFormCount: integer;
    function CreateForm(cls: TFormClass; const parentDivId, formName: string; clr: TColor): TWebHTMLDiv;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

{ TForm11 }

//------------------------------------------------------------------------------
function TMainForm.CreateForm(cls: TFormClass; const parentDivId,
  formName: string; clr: TColor): TWebHTMLDiv;
var
  frm: TWebForm;
  newId: string;
begin
  // -- Create the <div> that will hold the form. This div should in turn be a
  // -- child of 'forms_div'. This gives us flexibility for the placement of our
  // -- forms container div in the page body.
  result := TWebHTMLDiv.Create(FormsDiv);
  result.Parent := FormsDiv;
  result.ElementPosition := epAbsolute;

  // -- Set the ElementID. We need the div ElementID in the asm block but we
  // -- can simply use the WebCore-generated ID in this case. Setting our own
  // -- ID doesn't appear to cause any problems though.
  result.ElementID := parentDivId;
  //newId := result.ElementID;     // -- Get the WebCore-generated ID.

  asm
    var cntnr = document.getElementById(parentDivId);
    var frmDiv = document.getElementById("forms_div");

    if(frmDiv.childElementCount > 0){
      frmDiv.firstChild.insertAdjacentElement('beforebegin', cntnr);
    }else{
      frmDiv.appendChild(cntnr);
    }

    setTimeout(() => { window.dispatchEvent(new Event('resize')); }, 0);
  end;

  result.ElementHandle.style.setProperty('width', '100%');
  result.ElementHandle.style.setProperty('height', '100%');

  // -- Use the alpha channel to get opacity without affecting child elements.
  // -- We need this because setting the z-index isn't enough to hide the
  // -- low-z elements.
  result.ElementHandle.style.setProperty('background', 'rgba(255, 255, 255, 1.0)');

  Application.CreateFormDirect(cls, frm);
  // -- Use FormContainer to set the parent container. Using frm.Parent gives an error.
  frm.FormContainer := result.ElementID;
  frm.Align := alClient;
  frm.Color := clr;

  FFormList.Add(formName, result);
  FFormCount := FFormList.Count;

  // -- Use the z-index to control visibility of the form's parent div.
  // -- Toggling the Visible property caused element properties to be lost
  // -- (e.g. Width/Height) and was more work (more investigation required).
  result.ElementHandle.style.setProperty('z-index', IntToStr(FFormCount));

  FCurrentFormDiv := result;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebButton1Click(Sender: TObject);
var
  formDiv: TWebHTMLDiv;
  zIndex: string;
begin
  if not FFormList.TryGetValue('Form1', formDiv) then
    FCurrentFormDiv := CreateForm(TForm1, 'form1_parent', 'Form1', clYellow)
  else
  begin
    if assigned(FCurrentFormDiv) then
    begin
      zIndex := formDiv.ElementHandle.style.getPropertyValue('z-index');
      FCurrentFormDiv.ElementHandle.style.setProperty('z-index', zIndex);
    end;

    FCurrentFormDiv := formDiv;
    FCurrentFormDiv.ElementHandle.style.setProperty('z-index', IntToStr(FFormCount));
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebButton2Click(Sender: TObject);
var
  formDiv: TWebHTMLDiv;
  zIndex: string;
begin
  if not FFormList.TryGetValue('Form2', formDiv) then
    FCurrentFormDiv := CreateForm(TForm2, 'form2_parent', 'Form2', clBlue)
  else
  begin
    if assigned(FCurrentFormDiv) then
    begin
      zIndex := formDiv.ElementHandle.style.getPropertyValue('z-index');
      FCurrentFormDiv.ElementHandle.style.setProperty('z-index', zIndex);
    end;

    FCurrentFormDiv := formDiv;
    FCurrentFormDiv.ElementHandle.style.setProperty('z-index', IntToStr(FFormCount));
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebButton3Click(Sender: TObject);
var
  formDiv: TWebHTMLDiv;
  zIndex: string;
begin
  if not FFormList.TryGetValue('Form3', formDiv) then
    FCurrentFormDiv := CreateForm(TForm3, 'form3_parent', 'Form3', clGreen)
  else
  begin
    if assigned(FCurrentFormDiv) then
    begin
      zIndex := formDiv.ElementHandle.style.getPropertyValue('z-index');
      FCurrentFormDiv.ElementHandle.style.setProperty('z-index', zIndex);
    end;

    FCurrentFormDiv := formDiv;
    FCurrentFormDiv.ElementHandle.style.setProperty('z-index', IntToStr(FFormCount));
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebButton4Click(Sender: TObject);
var
  formDiv: TWebHTMLDiv;
  zIndex: string;
begin
  if not FFormList.TryGetValue('Form4', formDiv) then
    FCurrentFormDiv := CreateForm(TForm4, 'form4_parent', 'Form4', clRed)
  else
  begin
    if assigned(FCurrentFormDiv) then
    begin
      zIndex := formDiv.ElementHandle.style.getPropertyValue('z-index');
      FCurrentFormDiv.ElementHandle.style.setProperty('z-index', zIndex);
    end;

    FCurrentFormDiv := formDiv;
    FCurrentFormDiv.ElementHandle.style.setProperty('z-index', IntToStr(FFormCount));
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.WebFormCreate(Sender: TObject);
begin
  FFormList := TFormList.Create{([doOwnsValues])};
end;

end.